/*
 * Segment 2 (KEx),  SWE 2.DE Statischer Import MIF
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.mif.mif;

import org.geotools.feature.Feature;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

import de.bsvrz.kex.mif.konfigdaten.ImportException;

/**
 * .
 * @author BitCtrl Systems GmbH, Gieseler
 */
public final class MifTyp {
	/**
	 * Punktobjekt.
	 */
	public static final MifTyp MIF_PUNKT = new MifTyp("Punkt", "typ.punktXY");

	/**
	 * Linie.
	 */
	public static final MifTyp MIF_LINIE = new MifTyp("Linie", "typ.linieXY");

	/**
	 * Polylinie.
	 */
	public static final MifTyp MIF_POLYLINIE = new MifTyp("Polylinie", "typ.komplexXY");

	/**
	 * Bereich.
	 */
	public static final MifTyp MIF_BEREICH = new MifTyp("Bereich", "typ.komplexXY");

	/**
	 * Bogen.
	 */
	public static final MifTyp MIF_BOGEN = new MifTyp("Bogen", "typ.linieXY");

	/**
	 * Text.
	 */
	public static final MifTyp MIF_TEXT = new MifTyp("Text", "typ.punktXY");

	/**
	 * Attributname f&uuml;r den Text.
	 */
	public static final String MIF_TEXT_ATTRIBUT_NAME = "MIF_TEXT";

	/**
	 * Rechteck.
	 */
	public static final MifTyp MIF_RECHECK = new MifTyp("Rechteck", "typ.flächeXY");

	/**
	 * Abgerundetes Rechteck.
	 */
	public static final MifTyp MIF_ABGERUNDETES_RECHTECK = new MifTyp("abgerundetes Rechteck", "typ.flächeXY");

	/**
	 * Ellipse.
	 */
	public static final MifTyp MIF_ELLIPSE = new MifTyp("Ellipse", "typ.flächeXY");

	private final String name;

	private final String davtyp;

	private MifTyp(final String name, final String davtyp) {
		this.name = name;
		this.davtyp = davtyp;
	}

	/**
	 * .
	 * @return DAV-Typname
	 */
	public String getDavTyp() {
		return davtyp;
	}

	@Override
	public String toString() {
		return name;
	}

	/**
	 * Bestimmt den Typ des Features.
	 *
	 * @param f
	 *            Feature
	 * @return {@link MifTyp}
	 * @throws ImportException
	 *             wenn der Typ nicht bestimmt werden kann
	 */
	public static MifTyp bestimmeTyp(final Feature f) throws ImportException {
		final Object g = f.getAttribute(0);

		if (g instanceof Geometry) {
			final Geometry ge = (Geometry) g;
			try {
				return MifTyp.bestimmeTyp(ge);
			} catch (final ImportException ie) {
				throw new ImportException("Die Geometrie des Features '" + f.getID() + "' kann nicht bestimmt werden ("
						+ ie.getMessage() + ")");
			}
		}

		throw new ImportException("Die Geometrie des Features '" + f.getID() + "' kann nicht bestimmt werden");
	}

	/**
	 * Bestimmt den MIF-Typ aus der Geometrie.
	 *
	 * @param geometrie
	 *            Geometrie des Features
	 * @return {@link MifTyp}
	 * @throws ImportException
	 *             wenn der Typ nicht bestimmt werden kann
	 */
	public static MifTyp bestimmeTyp(final Geometry geometrie) throws ImportException {
		if (geometrie instanceof Point) {
			return MifTyp.MIF_PUNKT;
		}
		if (geometrie instanceof LineString) {
			return MifTyp.MIF_LINIE;
		}
		if (geometrie instanceof MultiPolygon) {
			return MifTyp.MIF_BEREICH;
		}
		if (geometrie instanceof Polygon) {
			return MifTyp.MIF_POLYLINIE;
		}
		if (geometrie instanceof MultiPoint) {
			return MifTyp.MIF_RECHECK;
		}

		throw new ImportException(
				"Der Typ der Geometrie '" + geometrie.getGeometryType() + "' kann nicht bestimmt werden");
	}

	/**
	 * Bestimmt den MIF-Typ.
	 * 
	 * @param name Name
	 * @return {@link MifTyp}
	 * @throws ImportException bei Fehlern
	 */
	public static MifTyp bestimmeTyp(final String name) throws ImportException {
		if ("Punkt".equalsIgnoreCase(name)) {
			return MifTyp.MIF_PUNKT;
		}
		if ("Linie".equalsIgnoreCase(name)) {
			return MifTyp.MIF_LINIE;
		}
		if ("Bereich".equalsIgnoreCase(name)) {
			return MifTyp.MIF_BEREICH;
		}
		if ("Polygon".equalsIgnoreCase(name)) {
			return MifTyp.MIF_POLYLINIE;
		}
		if ("Rechteck".equalsIgnoreCase(name)) {
			return MifTyp.MIF_RECHECK;
		}

		throw new ImportException("Der MIF-Typ kann aus dem Namen'" + name + "' nicht bestimmt werden");
	}
}
